@@ -4,7 +4,7 @@
   All Rights Reserved.
 
   This software is subject to the provisions of the Zope Public License,
-  Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
+  Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
   THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
   WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
   WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
@@ -12,10 +12,10 @@
 
  ****************************************************************************/
 
-#include "ExtensionClass.h"
+#include "ExtensionClass/ExtensionClass.h"
 
 #define _IN_ACQUISITION_C
-#include "Acquisition.h"
+#include "Acquisition/Acquisition.h"
 
 static ACQUISITIONCAPI AcquisitionCAPI;
 
@@ -37,8 +37,9 @@ static PyObject *py__add__, *py__sub__, *py__mul__, *py__div__,
   *py__pos__, *py__abs__, *py__nonzero__, *py__invert__, *py__int__,
   *py__long__, *py__float__, *py__oct__, *py__hex__,
   *py__getitem__, *py__setitem__, *py__delitem__,
-  *py__getslice__, *py__setslice__, *py__delslice__,
-  *py__len__, *py__of__, *py__call__, *py__repr__, *py__str__, *py__cmp__;
+  *py__getslice__, *py__setslice__, *py__delslice__,  *py__contains__,
+  *py__len__, *py__of__, *py__call__, *py__repr__, *py__str__, *py__cmp__,
+  *py__iter__;
 
 static PyObject *Acquired=0;
 
@@ -75,13 +76,15 @@ init_py_names(void)
   INIT_PY_NAME(__getslice__);
   INIT_PY_NAME(__setslice__);
   INIT_PY_NAME(__delslice__);
+  INIT_PY_NAME(__contains__);
   INIT_PY_NAME(__len__);
   INIT_PY_NAME(__of__);
   INIT_PY_NAME(__call__);
   INIT_PY_NAME(__repr__);
   INIT_PY_NAME(__str__);
   INIT_PY_NAME(__cmp__);
-  
+  INIT_PY_NAME(__iter__);
+
 #undef INIT_PY_NAME
 }
 
@@ -804,6 +807,24 @@ Wrapper_ass_slice(Wrapper *self, int  ilow, int  ihigh, PyObject *v)
   return 0;
 }
 
+static int
+Wrapper_contains(Wrapper *self, PyObject *v)
+{
+  long c;
+
+  UNLESS(v=CallMethodO(OBJECT(self),py__contains__,Build("(O)", v) ,NULL))
+    return -1;
+  c = PyInt_AsLong(v);
+  Py_DECREF(v);
+  return c;
+}
+
+static PyObject * 
+Wrapper_iter(Wrapper *self)
+{
+  return PyObject_GetIter(self->obj);
+}
+
 static PySequenceMethods Wrapper_as_sequence = {
 	(inquiry)Wrapper_length,		/*sq_length*/
 	(binaryfunc)Wrapper_add,		/*sq_concat*/
@@ -812,6 +833,7 @@ static PySequenceMethods Wrapper_as_sequence = {
 	(intintargfunc)Wrapper_slice,		/*sq_slice*/
 	(intobjargproc)Wrapper_ass_item,	/*sq_ass_item*/
 	(intintobjargproc)Wrapper_ass_slice,	/*sq_ass_slice*/
+	(objobjproc)Wrapper_contains,		/*sq_contains*/
 };
 
 /* -------------------------------------------------------------- */
@@ -1208,7 +1230,7 @@ static PyExtensionClass Wrappertype = {
   /* tp_clear          */ (inquiry)Wrapper_clear,
   /* tp_richcompare    */ (richcmpfunc)0,
   /* tp_weaklistoffset */ (long)0,
-  /* tp_iter           */ (getiterfunc)0,
+  (getiterfunc)Wrapper_iter,		/*tp_iter*/
   /* tp_iternext       */ (iternextfunc)0,
   /* tp_methods        */ Wrapper_methods,
   /* tp_members        */ 0,
@@ -1252,7 +1274,7 @@ static PyExtensionClass XaqWrappertype = {
   /* tp_clear          */ (inquiry)Wrapper_clear,
   /* tp_richcompare    */ (richcmpfunc)0,
   /* tp_weaklistoffset */ (long)0,
-  /* tp_iter           */ (getiterfunc)0,
+  (getiterfunc)Wrapper_iter,		/*tp_iter*/
   /* tp_iternext       */ (iternextfunc)0,
   /* tp_methods        */ Wrapper_methods,
   /* tp_members        */ 0,
@@ -1616,7 +1638,7 @@ init_Acquisition(void)
   /* Create the module and add the functions */
   m = Py_InitModule4("_Acquisition", methods,
 	   "Provide base classes for acquiring objects\n\n"
-	   "$Id: _Acquisition.c 38586 2005-09-24 12:10:07Z andreasjung $\n",
+	   "$Id: _Acquisition.c 99193 2009-04-15 14:44:25Z witsch $\n",
 		     OBJECT(NULL),PYTHON_API_VERSION);
 
   d = PyModule_GetDict(m);
