@@ -128,6 +128,9 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         res = self._newPS('return 1 * 5 + 4 / 2 - 6')()
         self.assertEqual(res, 1)
 
+    def testCollector2295(self):
+        res = self._newPS('if False:\n  pass\n#hi')
+
     def testReduce(self):
         res = self._newPS('return reduce(lambda x, y: x + y, [1,3,5,7])')()
         self.assertEqual(res, 16)
@@ -197,6 +200,9 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         bound = f.__render_with_namespace__({'yes': 1, 'no': self.fail})
         self.assertEqual(bound, 1)
 
+    def testNSBindInvalidHeader(self):
+        self.assertRaises(SyntaxError, self._filePS, 'ns_bind_invalid')
+
     def testBooleanMap(self):
         res = self._filePS('boolean_map')()
         self.failUnless(res)
@@ -205,6 +211,14 @@ class TestPythonScriptNoAq(PythonScriptTestBase):
         f = self._filePS('complex_print')
         self.assertEqual(f.get_size(), len(f.read()))
 
+    def testRaiseSystemExitLaunchpad257269(self):
+        ps = self._newPS("raise SystemExit")
+        self.assertRaises(ValueError, ps)
+
+    def testEncodingTestDotTestAllLaunchpad257276(self):
+        ps = self._newPS("return 'foo'.encode('test.testall')")
+        self.assertRaises(LookupError, ps)
+
 
 class TestPythonScriptErrors(PythonScriptTestBase):
     
@@ -222,9 +236,10 @@ class TestPythonScriptErrors(PythonScriptTestBase):
         self.assertPSRaises(SyntaxError, path='subversive_except')
 
     def testBadImports(self):
-        self.assertPSRaises(ImportError, body="from string import *")
-        self.assertPSRaises(ImportError, body="from datetime import datetime")
-        #self.assertPSRaises(ImportError, body="import mmap")
+        from zExceptions import Unauthorized
+        self.assertPSRaises(Unauthorized, body="from string import *")
+        self.assertPSRaises(Unauthorized, body="from datetime import datetime")
+        self.assertPSRaises(Unauthorized, body="import mmap")
 
     def testAttributeAssignment(self):
         # It's illegal to assign to attributes of anything that
