@@ -1,7 +1,7 @@
 # Author: David Goodger
 # Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1.2.10.8 $
-# Date: $Date: 2005/10/09 14:43:43 $
+# Revision: $Revision: 4242 $
+# Date: $Date: 2006-01-06 00:28:53 +0100 (Fri, 06 Jan 2006) $
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -26,11 +26,9 @@ __docformat__ = 'reStructuredText'
 import sys
 import os
 import re
-import copy
 import warnings
-import xml.dom.minidom
 from types import IntType, SliceType, StringType, UnicodeType, \
-     TupleType, ListType
+     TupleType, ListType, ClassType
 from UserString import UserString
 
 
@@ -65,8 +63,10 @@ class Node:
         """
         return 1
 
-    def asdom(self, dom=xml.dom.minidom):
+    def asdom(self, dom=None):
         """Return a DOM **fragment** representation of this Node."""
+        if dom is None:
+            import xml.dom.minidom as dom
         domroot = dom.Document()
         return self._dom_node(domroot)
 
@@ -82,6 +82,10 @@ class Node:
         """Return a copy of self."""
         raise NotImplementedError
 
+    def deepcopy(self):
+        """Return a deep copy of self (also copying children)."""
+        raise NotImplementedError
+
     def setup_child(self, child):
         child.parent = self
         if self.document:
@@ -175,6 +179,11 @@ class Node:
         * the siblings of the parent (if ascend is true) and their
           descendants (if also descend is true), and so on
 
+        If `condition` is not None, the iterable contains only nodes
+        for which ``condition(node)`` is true.  If `condition` is a
+        node class ``cls``, it is equivalent to a function consisting
+        of ``return isinstance(node, cls)``.
+
         If ascend is true, assume siblings to be true as well.
 
         For example, given the following tree::
@@ -198,6 +207,10 @@ class Node:
         r = []
         if ascend:
             siblings=1
+        if isinstance(condition, ClassType):
+            node_class = condition
+            def condition(node, node_class=node_class):
+                return isinstance(node, node_class)
         if include_self and (condition is None or condition(self)):
             r.append(self)
         if descend and len(self.children):
@@ -219,7 +232,6 @@ class Node:
                     node = node.parent
         return r
 
-
     def next_node(self, condition=None,
                   include_self=0, descend=1, siblings=0, ascend=0):
         """
@@ -280,6 +292,9 @@ class Text(Node, UserString):
     def copy(self):
         return self.__class__(self.data)
 
+    def deepcopy(self):
+        return self.copy()
+
     def pformat(self, indent='    ', level=0):
         result = []
         indent = indent * level
@@ -325,9 +340,9 @@ class Element(Node):
     This is equivalent to ``element.extend([node1, node2])``.
     """
 
-    attr_defaults = {'ids': [], 'classes': [], 'names': [],
-                     'dupnames': [], 'backrefs': []}
-    """Default attributes."""
+    list_attributes = ('ids', 'classes', 'names', 'dupnames', 'backrefs')
+    """List attributes, automatically initialized to empty lists for
+    all nodes."""
 
     tagname = None
     """The element generic identifier. If None, it is set as an instance
@@ -345,11 +360,20 @@ class Element(Node):
 
         self.extend(children)           # maintain parent info
 
-        self.attributes = copy.deepcopy(self.attr_defaults)
+        self.attributes = {}
         """Dictionary of attribute {name: value}."""
 
+        # Initialize list attributes.
+        for att in self.list_attributes:
+            self.attributes[att] = []
+
         for att, value in attributes.items():
-            self.attributes[att.lower()] = value
+            att = att.lower()
+            if att in self.list_attributes:
+                # mutable list; make a copy for this node
+                self.attributes[att] = value[:]
+            else:
+                self.attributes[att] = value
 
         if self.tagname is None:
             self.tagname = self.__class__.__name__
@@ -358,7 +382,7 @@ class Element(Node):
         element = domroot.createElement(self.tagname)
         for attribute, value in self.attlist():
             if isinstance(value, ListType):
-                value = ' '.join(['%s' % v for v in value])
+                value = ' '.join([serial_escape('%s' % v) for v in value])
             element.setAttribute(attribute, '%s' % value)
         for child in self.children:
             element.appendChild(child._dom_node(domroot))
@@ -401,7 +425,7 @@ class Element(Node):
             if value is None:           # boolean attribute
                 parts.append(name)
             elif isinstance(value, ListType):
-                values = ['%s' % v for v in value]
+                values = [serial_escape('%s' % v) for v in value]
                 parts.append('%s="%s"' % (name, ' '.join(values)))
             else:
                 parts.append('%s="%s"' % (name, value))
@@ -527,11 +551,23 @@ class Element(Node):
         return self.children.index(item)
 
     def is_not_default(self, key):
-        try:
-            return self[key] != self.attr_defaults[key]
-        except KeyError:
+        if self[key] == [] and key in self.list_attributes:
+            return 0
+        else:
             return 1
 
+    def update_basic_atts(self, dict):
+        """
+        Update basic attributes ('ids', 'names', 'classes',
+        'dupnames', but not 'source') from node or dictionary `dict`.
+        """
+        if isinstance(dict, Node):
+            dict = dict.attributes
+        for att in ('ids', 'classes', 'names', 'dupnames'):
+            for value in dict.get(att, []):
+                if not value in self[att]:
+                    self[att].append(value)
+
     def clear(self):
         self.children = []
 
@@ -544,6 +580,28 @@ class Element(Node):
         elif new is not None:
             self[index:index+1] = new
 
+    def replace_self(self, new):
+        """
+        Replace `self` node with `new`, where `new` is a node or a
+        list of nodes.
+        """
+        update = new
+        if not isinstance(new, Node):
+            # `new` is a list; update first child.
+            try:
+                update = new[0]
+            except IndexError:
+                update = None
+        if isinstance(update, Element):
+            update.update_basic_atts(self)
+        else:
+            # `update` is a Text node or `new` is an empty list.
+            # Assert that we aren't losing any attributes.
+            for att in ('ids', 'names', 'classes', 'dupnames'):
+                assert not self[att], \
+                       'Losing "%s" attribute: %s' % (att, self[att])
+        self.parent.replace(self, new)
+
     def first_child_matching_class(self, childclass, start=0, end=sys.maxint):
         """
         Return the index of the first child whose class exactly matches.
@@ -593,6 +651,11 @@ class Element(Node):
     def copy(self):
         return self.__class__(**self.attributes)
 
+    def deepcopy(self):
+        copy = self.copy()
+        copy.extend([child.deepcopy() for child in self.children])
+        return copy
+
     def set_class(self, name):
         """Add a new class to the "classes" attribute."""
         warnings.warn('docutils.nodes.Element.set_class deprecated; '
@@ -708,10 +771,16 @@ class Inline: pass
 
 class Referential(Resolvable): pass
 
+
 class Targetable(Resolvable):
 
     referenced = 0
 
+    indirect_reference_name = None
+    """Holds the whitespace_normalized_name (contains mixed case) of a target.
+    Required for MoinMoin/reST compatibility."""
+
+
 class Labeled:
     """Contains a `label` as its first element."""
 
@@ -722,6 +791,13 @@ class Labeled:
 
 class document(Root, Structural, Element):
 
+    """
+    The document root element.
+
+    Do not instantiate this class directly; use
+    `docutils.utils.new_document()` instead.
+    """
+
     def __init__(self, settings, reporter, *args, **kwargs):
         Element.__init__(self, *args, **kwargs)
 
@@ -737,12 +813,6 @@ class document(Root, Structural, Element):
         self.reporter = reporter
         """System message generator."""
 
-        self.external_targets = []
-        """List of external target nodes."""
-
-        self.internal_targets = []
-        """List of internal target nodes."""
-
         self.indirect_targets = []
         """List of indirect target nodes."""
 
@@ -769,22 +839,12 @@ class document(Root, Structural, Element):
         self.ids = {}
         """Mapping of ids to nodes."""
 
-        self.substitution_refs = {}
-        """Mapping of substitution names to lists of substitution_reference
-        nodes."""
-
         self.footnote_refs = {}
         """Mapping of footnote labels to lists of footnote_reference nodes."""
 
         self.citation_refs = {}
         """Mapping of citation labels to lists of citation_reference nodes."""
 
-        self.anonymous_targets = []
-        """List of anonymous target nodes."""
-
-        self.anonymous_refs = []
-        """List of anonymous reference nodes."""
-
         self.autofootnotes = []
         """List of auto-numbered footnote nodes."""
 
@@ -827,8 +887,10 @@ class document(Root, Structural, Element):
 
         self.document = self
 
-    def asdom(self, dom=xml.dom.minidom):
+    def asdom(self, dom=None):
         """Return a DOM representation of this document."""
+        if dom is None:
+            import xml.dom.minidom as dom
         domroot = dom.Document()
         domroot.appendChild(self._dom_node(domroot))
         return domroot
@@ -862,16 +924,17 @@ class document(Root, Structural, Element):
 
         The following state transition table shows how `self.nameids` ("ids")
         and `self.nametypes` ("types") change with new input (a call to this
-        method), and what actions are performed:
+        method), and what actions are performed ("implicit"-type system
+        messages are INFO/1, and "explicit"-type system messages are ERROR/3):
 
         ====  =====  ========  ========  =======  ====  =====  =====
          Old State    Input          Action        New State   Notes
         -----------  --------  -----------------  -----------  -----
         ids   types  new type  sys.msg.  dupname  ids   types
         ====  =====  ========  ========  =======  ====  =====  =====
-        --    --     explicit  --        --       new   True
-        --    --     implicit  --        --       new   False
-        None  False  explicit  --        --       new   True
+        -     -      explicit  -         -        new   True
+        -     -      implicit  -         -        new   False
+        None  False  explicit  -         -        new   True
         old   False  explicit  implicit  old      new   True
         None  True   explicit  explicit  new      None  True
         old   True   explicit  explicit  new,old  None  True   [#]_
@@ -952,12 +1015,6 @@ class document(Root, Structural, Element):
     def note_refid(self, node):
         self.refids.setdefault(node['refid'], []).append(node)
 
-    def note_external_target(self, target):
-        self.external_targets.append(target)
-
-    def note_internal_target(self, target):
-        self.internal_targets.append(target)
-
     def note_indirect_target(self, target):
         self.indirect_targets.append(target)
         if target['names']:
@@ -965,10 +1022,6 @@ class document(Root, Structural, Element):
 
     def note_anonymous_target(self, target):
         self.set_id(target)
-        self.anonymous_targets.append(target)
-
-    def note_anonymous_ref(self, ref):
-        self.anonymous_refs.append(ref)
 
     def note_autofootnote(self, footnote):
         self.set_id(footnote)
@@ -1005,7 +1058,6 @@ class document(Root, Structural, Element):
 
     def note_substitution_def(self, subdef, def_name, msgnode=None):
         name = whitespace_normalize_name(def_name)
-        subdef['names'].append(name)
         if self.substitution_defs.has_key(name):
             msg = self.reporter.error(
                   'Duplicate substitution definition name: "%s".' % name,
@@ -1020,8 +1072,7 @@ class document(Root, Structural, Element):
         self.substitution_names[fully_normalize_name(name)] = name
 
     def note_substitution_ref(self, subref, refname):
-        name = subref['refname'] = whitespace_normalize_name(refname)
-        self.substitution_refs.setdefault(name, []).append(subref)
+        subref['refname'] = whitespace_normalize_name(refname)
 
     def note_pending(self, pending, priority=None):
         self.transformer.add_pending(pending, priority)
@@ -1148,6 +1199,7 @@ class transition(Structural, Element): pass
 
 class paragraph(General, TextElement): pass
 class compound(General, Element): pass
+class container(General, Element): pass
 class bullet_list(Sequential, Element): pass
 class enumerated_list(Sequential, Element): pass
 class list_item(Part, Element): pass
@@ -1230,6 +1282,13 @@ class entry(Part, Element): pass
 
 class system_message(Special, BackLinkable, PreBibliographic, Element):
 
+    """
+    System message element.
+
+    Do not instantiate this class directly; use
+    ``document.reporter.info/warning/error/severe()`` instead.
+    """
+
     def __init__(self, message=None, *children, **attributes):
         if message:
             p = paragraph('', message)
@@ -1363,7 +1422,7 @@ node_class_names = """
         authors
     block_quote bullet_list
     caption caution citation citation_reference classifier colspec comment
-        compound contact copyright
+        compound contact container copyright
     danger date decoration definition definition_list definition_list_item
         description docinfo doctest_block document
     emphasis entry enumerated_list error
@@ -1682,3 +1741,15 @@ def fully_normalize_name(name):
 def whitespace_normalize_name(name):
     """Return a whitespace-normalized name."""
     return ' '.join(name.split())
+
+def serial_escape(value):
+    """Escape string values that are elements of a list, for serialization."""
+    return value.replace('\\', r'\\').replace(' ', r'\ ')
+
+# 
+#
+# Local Variables:
+# indent-tabs-mode: nil
+# sentence-end-double-space: t
+# fill-column: 78
+# End:
